@@ -1,5 +1,5 @@
 import numpy as np
-from typing import Tuple
+from typing import Callable, Tuple
 
 def _parseAxis(A:np.ndarray, axis:Tuple[int,int]):
     assert all([-A.ndim <= x < A.ndim for x in axis]), "Specified dimension must respect -X.ndim <= ParmDim < X.ndim"
@@ -170,4 +170,28 @@ def FFTGuess(x:np.ndarray, y:np.ndarray, axis=-1) -> Tuple[float, float, float,
     if peakEnergy/totalEnergy < 0.5:
         print("Warning: the most prominent peak does not account for more than half the energy, the FFTGuess may not be the best one. ")
 
-    return amp, omega, phi, voff
\ No newline at end of file
+    return amp, omega, phi, voff
+
+def NumericalJacobian(f:Callable, x: np.ndarray, parms:Tuple[float, ...], dx:float = 1e-8) -> np.ndarray:
+    """
+    Computes a numerical jacobian on the provided function using central differences.
+    Input parameters:
+        f:  FitFunc(self, x:np.ndarray, *args:List[float]) -> np.ndarray
+        x:  the array of independant variable
+        parms:  a tuple containing the parameters
+        dx:     how much to nudge the function in each direction to get a gradient
+
+    Output parameters:
+        the jacobian matrix, a numpy.ndarray of shape [x.size, len(parms)]
+    """
+
+    jac = np.zeros((x.size, len(parms)))
+
+    for i, parm in enumerate(parms):
+        parm_plus = list(parms)
+        parm_plus[i] += dx
+        parm_minus = list(parms)
+        parm_minus[i] -= dx
+        jac[:,i] = (f(x, *parm_plus) - f(x, *parm_minus))/(2*dx)
+
+    return jac
\ No newline at end of file
